@@ -152,12 +152,12 @@ static time_t mktime_from_rfc2616(const char *date) {
 /*
  * SYNOPSIS:
  *   double set_clock(
- *                    time_t timeval,
+ *                    struct timeval *tvinfo,
  *                    int allow_adj
  *                   );
  *
  * ARGUMENTS:
- *   time_t timeval          Time value to set clock to. (GMT/UTC)
+ *   struct timeval *tvinfo  Pointer to time value to set clock to. (GMT/UTC)
  *   int allow_adj           Set to 1 if you want to allow time to be "Slew"'d
  *                           if the OS supports this.
  *
@@ -170,49 +170,38 @@ static time_t mktime_from_rfc2616(const char *date) {
  *   value as if no adjustment was needed.
  *
  */
-double set_clock(time_t timeval, int allow_adj) {
-	struct timeval tvinfo;
+double set_clock(struct timeval *tvinfo, int allow_adj) {
 	struct timeval tvdelta;
-	time_t currtime;
-	double retval;
+	double retval = 0.0;
 	int chkret = -1;
 
 	gettimeofday(&tvdelta, NULL);
 
-	currtime = tvdelta.tv_sec;
-
 #ifdef HAVE_ADJTIME
 	if (allow_adj) {
-		tvdelta.tv_sec = timeval - tvdelta.tv_sec;
-		tvdelta.tv_usec = 0;
+		tvdelta.tv_sec = tvinfo->tv_sec - tvdelta.tv_sec;
+		tvdelta.tv_usec = tvinfo->tv_usec - tvdelta.tv_usec;
 	}
 
 	if ((tvdelta.tv_sec < 60 && tvdelta.tv_sec > -60) && allow_adj) {
 		chkret = adjtime(&tvdelta, NULL);
-#ifdef HAVE_SYSLOG
 		if (chkret >= 0) {
-			syslog(LOG_NOTICE, "Adjusting clock by %f seconds (slew).", (double) tvdelta.tv_sec + (((double) tvdelta.tv_usec) / 1000000.0));
-		}
+			retval = tvdelta.tv_sec + (((double) tvdelta.tv_usec) / 1000000.0);
+#ifdef HAVE_SYSLOG
+			syslog(LOG_NOTICE, "Adjusting clock by %f seconds (slew).", retval);
 #endif
+		}
 	}
 #endif
 
 	if (chkret < 0) {
-		tvinfo.tv_sec = timeval;
-		tvinfo.tv_usec = 0;
-
-		chkret = settimeofday(&tvinfo, NULL);
-#ifdef HAVE_SYSLOG
+		chkret = settimeofday(tvinfo, NULL);
 		if (chkret >= 0) {
-			syslog(LOG_NOTICE, "Adjusting clock by %f seconds (step).", (double) (tvinfo.tv_sec - currtime) + (((double) tvinfo.tv_usec) / 1000000.0));
-		}
+			retval = (tvinfo->tv_sec - tvdelta.tv_sec) + (((double) (tvinfo->tv_usec - tvdelta.tv_usec)) / 1000000.0);
+#ifdef HAVE_SYSLOG
+			syslog(LOG_NOTICE, "Adjusting clock by %f seconds (step).", retval);
 #endif
-	}
-
-	if (chkret < 0) {
-		retval = 0.0;
-	} else {
-		retval = currtime - timeval;
+		}
 	}
 
 	return(retval);
@@ -372,37 +361,46 @@ int htp_init(void) {
 
 /*
  * SYNOPSIS:
- *   int compare_time(
- *                    const void *a,
- *                    const void *b
- *                   );
+ *   static int compare_time(
+ *                           const void *a,
+ *                           const void *b
+ *                          );
  *
  * ARGUMENTS:
- *   const void *a           A pointer to a `time_t' value.
- *   const void *b           Another pointer to a `time_t' value.
+ *   const void *a           A pointer to a `struct timeval' value.
+ *   const void *b           Another pointer to a `struct timeval' value.
  *
  * RETURN VALUE:
- *   This function returns -1, 0, or 1 if the `time_t' value pointed to by a
- *   is less than, equal to, or greater than (respectively) the `time_t'
- *   value pointed to be b.
+ *   This function returns -1, 0, or 1 if the `struct timeval' value pointed
+ *   to by a is less than, equal to, or greater than (respectively) the
+ *   `struct timeval' value pointed to be b.
  *
  * NOTES:
- *   This function is meant to be passed as the 4th argument to qsort().
+ *   This function is meant to be passed as the 4th (func) argument to
+ *   qsort().
  *
  */
-int compare_time(const void *a, const void *b) {
-	const time_t *tmpptr = NULL;
-	time_t  timea = 0, timeb = 0;
+static int compare_time(const void *a, const void *b) {
+	const struct timeval *timea = 0, *timeb = 0;
 
-	tmpptr = a;
-	timea = *tmpptr;
-	tmpptr = b;
-	timeb = *tmpptr;
+	timea = a;
+	timeb = b;
 
-	if (timea < timeb) {
+	if (timea->tv_sec < timeb->tv_sec) {
 		return(-1);
 	}
-	if (timea > timeb) {
+
+	if (timea->tv_sec > timeb->tv_sec) {
+		return(1);
+	}
+
+	/* timea->tv_sec MUST EQUAL timeb->tv_sec. */
+
+	if (timea->tv_usec < timeb->tv_usec) {
+		return(-1);
+	}
+
+	if (timea->tv_usec > timeb->tv_usec) {
 		return(1);
 	}
 
@@ -411,12 +409,12 @@ int compare_time(const void *a, const void *b) {
 
 /*
  * SYNOPSIS:
- *   time_t htp_calctime(
- *                       struct timeserver_st *timeservers,
- *                       unsigned int totaltimeservers,
- *                       const char *proxyhost,
- *                       unsigned int proxyport
- *                      );
+ *   struct timeval *htp_calctime(
+ *                                struct timeserver_st *timeservers,
+ *                                unsigned int totaltimeservers,
+ *                                const char *proxyhost,
+ *                                unsigned int proxyport
+ *                               );
  *
  * ARGUMENTS:
  *   struct timeserver_st *timeservers      Pointer to the first element of
@@ -428,9 +426,8 @@ int compare_time(const void *a, const void *b) {
  *   unsigned int proxyport                 Port to connect to proxy on.
  *
  * RETURN VALUE:
- *   This function returns the best approximation of the current `time_t'
- *   value (in GMT/UTC) that can be deduced from the timeservers specified.
- *   -1 is returned on error.
+ *   This function returns the best approximation of the current time in
+ *   UTC/GMT.  NULL is returned on failure.
  *
  * NOTES:
  *   If more than 2 timeservers are specified the time values outside of one
@@ -440,89 +437,124 @@ int compare_time(const void *a, const void *b) {
  *   This function accounts for the time it takes for itself to execute in
  *   its return value.
  *
+ *   This function returns a static buffer, if you wish to have two calls
+ *   to it you must store the values elsewhere.
+ *
  */
-time_t htp_calctime(struct timeserver_st *timeservers, unsigned int totaltimeservers, const char *proxyhost, unsigned int proxyport) {
+struct timeval *htp_calctime(struct timeserver_st *timeservers, unsigned int totaltimeservers, const char *proxyhost, unsigned int proxyport) {
 	unsigned int timeind = 0, meantimeind = 0;
 	unsigned int totaltimes = 0, totalstddevtimes = 0;
 	unsigned int port = 0;
-	time_t timevals[128] = {0}, timeval = -1;
-	time_t mintime, maxtime = 0, avgtime = 0, meantime = 0;
-	time_t stddevavgtime = 0;
-	time_t tmptime = 0;
-	time_t newtimeval = 0;
-	time_t start_time = 0, offset_time = 0;
+	time_t timeval = -1;
+	static struct timeval retval;
+	struct timeval timevals[128];
+	struct timeval start_time, end_time;
+	struct timeval avgtime, stddevavgtime;
+	struct timeval *mintime, *maxtime, *meantime;
+	struct timeval *newtimeval = NULL;
+	struct timeval *tmptime;
+	long newusecs = 0;
 	char *host = NULL;
 	int stddevtimes = 0;
 
-	start_time = time(NULL);
-
-	mintime = start_time * 2;
-
-	totaltimes = 0;
+	if (gettimeofday(&start_time, NULL) < 0) {
+		return(NULL);
+	}
 
 	for (timeind = 0; timeind < totaltimeservers; timeind++) {	
 		host = timeservers[timeind].host;
 		port = timeservers[timeind].port;
+
 		timeval = get_server_time(host, port, proxyhost, proxyport);
 		if (timeval < 0) {
 			continue;
 		}
 
 		/* Remove the amount of time spent so far. */
-		offset_time = time(NULL) - start_time;
+		gettimeofday(&end_time, NULL);
+		timevals[totaltimes].tv_sec = timeval - (end_time.tv_sec - start_time.tv_sec);
+		newusecs = end_time.tv_usec - start_time.tv_usec;
+		while (newusecs < 0) {
+			newusecs += 1000000;
+			timevals[totaltimes].tv_sec--;
+		}
+		timevals[totaltimes].tv_usec = newusecs;
 
-		timevals[totaltimes] = timeval - offset_time;
 		totaltimes++;
 	}
 
-	meantimeind = totaltimes / 2;
-
+	/* Return failure if no time servers could be reached. */
 	if (totaltimes == 0) {
-		return(-1);
+		return(NULL);
 	}
 
+	/* Find the mean time index. */
+	meantimeind = totaltimes / 2;
+
 	/* If there's only one time, don't bother with anything else. */
 	if (totaltimes == 1) {
-		newtimeval = timevals[0];
+		newtimeval = &timevals[0];
 	} else {
 		qsort(&timevals, totaltimes, sizeof(timevals[0]), compare_time);
-		mintime = timevals[0];
-		maxtime = timevals[totaltimes - 1];
-		meantime = timevals[meantimeind];
+		mintime = &timevals[0];
+		maxtime = &timevals[totaltimes - 1];
+		meantime = &timevals[meantimeind];
+
+		avgtime.tv_sec = 0;
+		avgtime.tv_usec = 0;
 
 		for (timeind = 0; timeind < totaltimes; timeind++) {
-			tmptime = timevals[timeind];
-			avgtime += tmptime - mintime; /* This trickery is to avoid overflowing the type. */
+			tmptime = &timevals[timeind];
+			avgtime.tv_sec += tmptime->tv_sec - mintime->tv_sec; /* This trickery is to avoid overflowing the type. */
+			avgtime.tv_usec += tmptime->tv_usec;
 		}
-		avgtime /= totaltimes;
-		avgtime += mintime; /* Part of the same silly trickery. */
+		avgtime.tv_usec /= totaltimes;
+		avgtime.tv_sec /= totaltimes;
+		while (avgtime.tv_usec >= 1000000) {
+			avgtime.tv_sec++;
+			avgtime.tv_usec -= 1000000;
+		}
+		avgtime.tv_sec += mintime->tv_sec; /* Part of the same silly trickery. */
 
 		/* If there are only 2 values, we can do no more. */
 		if (totaltimes == 2) {
-			newtimeval = avgtime;
+			newtimeval = &avgtime;
 		} else {
+
+			/* Otherwise we only include values within one standard deviation */
 			stddevtimes = ((double) totaltimes) * 0.34; /* One standard deviation, +/- 34% about the mean. */
 
 			for (timeind = (meantimeind - stddevtimes); timeind < (meantimeind + stddevtimes); timeind++) {
-				tmptime = timevals[timeind];
-				stddevavgtime += tmptime - mintime;
+				tmptime = &timevals[timeind];
+				stddevavgtime.tv_sec += tmptime->tv_sec - mintime->tv_sec;
+				stddevavgtime.tv_usec += tmptime->tv_usec;
 				totalstddevtimes++;
 			}
-			stddevavgtime /= totalstddevtimes;
-			stddevavgtime += mintime;
+			stddevavgtime.tv_sec /= totalstddevtimes;
+			stddevavgtime.tv_usec /= totalstddevtimes;
+			while (stddevavgtime.tv_usec >= 1000000) {
+				stddevavgtime.tv_sec++;
+				stddevavgtime.tv_usec -= 1000000;
+			}
+			stddevavgtime.tv_sec += mintime->tv_sec;
 
-			newtimeval = stddevavgtime;
+			newtimeval = &stddevavgtime;
 		}
 	}
 
 	if (newtimeval <= 0) {
-		return(-1);
+		return(NULL);
 	}
 
-	offset_time = time(NULL) - start_time;
-
 	/* Add back the time this process took. */
-	newtimeval += offset_time;
+	gettimeofday(&end_time, NULL);
+	retval.tv_sec = newtimeval->tv_sec + (end_time.tv_sec - start_time.tv_sec);
+	newusecs = start_time.tv_usec + newtimeval->tv_usec;
+	while (newusecs >= 1000000) {
+		newusecs -= 1000000;
+		retval.tv_sec++;
+	}
+	retval.tv_usec = newusecs;
 
-	return(newtimeval);
+	return(&retval);
 }
