@@ -11,8 +11,8 @@ struct timeserver_st {
         unsigned int port;
 };
 
-double set_clock(time_t timeval, int allow_adj);
-time_t htp_calctime(struct timeserver_st *timeservers, unsigned int totaltimeservers, const char *proxyhost, unsigned int proxyport);
+double set_clock(struct timeval *tvinfo, int allow_adj);
+struct timeval *htp_calctime(struct timeserver_st *timeservers, unsigned int totaltimeservers, const char *proxyhost, unsigned int proxyport);
 int htp_init(void);
 
 #endif
