@@ -107,7 +107,7 @@ static char svcName[] = "HTP Time Daemon";
  *
  */
 int htpd_loop(void) {
-	time_t newtime = 0;
+	struct timeval *newtime;
 	long sleeptime;
 	double delta;
 #ifdef _USE_WIN32_
