@@ -26,11 +26,10 @@
 #endif
 #ifdef HAVE_TIME_H
 #include <time.h>
-#else
+#endif
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
-#endif
 
 #include "htp.h"
 
@@ -48,9 +47,11 @@ int main(int argc, char *argv[]) {
 	unsigned int argind = 0, timeind = 0;
 	unsigned int totaltimeservers = 0;
 	struct timeserver_st timeservers[128];
-	time_t newtime = 0;
+	struct timeval *newtime;
+	double delta = 0.0;
 	char *host = NULL, *portstr = NULL;
 	char *proxyhost = NULL, *proxyportstr = NULL;
+	char strtime[128];
 	int port = -1, proxyport = -1;
 	int allowadj = 0;
 	int queryonly = 0;
@@ -127,16 +128,18 @@ int main(int argc, char *argv[]) {
 
 	newtime = htp_calctime(timeservers, totaltimeservers, proxyhost, proxyport);
 
-	if (newtime < 0) {
+	if (newtime == NULL) {
 		fprintf(stderr, "No servers could be reached.\n");
 		return(EXIT_FAILURE);
 	}
 
 	if (!queryonly) {
-		set_clock(newtime, allowadj);
+		delta = set_clock(newtime, allowadj);
 	}
 
-	printf("%s", asctime(localtime(&newtime)));
+	strftime(strtime, sizeof(strtime), "%a %b %d %H:%M:%S %Y", localtime(&newtime->tv_sec));
+
+	printf("%s [+ %f] (delta: %f)\n", strtime, ((double) newtime->tv_usec) / 1000000.0, delta);
 
 	return(EXIT_SUCCESS);
 }
